@@ -1,7 +1,14 @@
 using SpecialFunctions, Documenter
+using DocumenterCitations
 
 # `using SpecialFunctions` for all doctests
 DocMeta.setdocmeta!(SpecialFunctions, :DocTestSetup, :(using SpecialFunctions); recursive=true)
+
+bib = CitationBibliography(
+       joinpath(@__DIR__, "src", "refs.bib");
+       style = :authoryear,
+)
+
 makedocs(modules=[SpecialFunctions],
          sitename="SpecialFunctions.jl",
          authors="Jeff Bezanson, Stefan Karpinski, Viral B. Shah, et al.",
@@ -9,7 +16,8 @@ makedocs(modules=[SpecialFunctions],
          pages=["Home" => "index.md",
                 "Overview" => "functions_overview.md",
                 "Reference" => "functions_list.md"],
-         #warnonly=[:missing_docs],
+         plugins=[bib],
+         checkdocs=:exports,
         )
 
 deploydocs(repo="github.com/JuliaMath/SpecialFunctions.jl.git")
