@@ -7,6 +7,8 @@ using Random
 using Test
 using Base.MathConstants: γ
 
+using Aqua: Aqua
+using ExplicitImports: ExplicitImports
 using JET: JET
 
 using SpecialFunctions: AmosException, f64
@@ -35,7 +37,8 @@ tests = [
     "logabsgamma",
     "sincosint",
     "other_tests",
-    "chainrules"
+    "chainrules",
+    "qa"
 ]
 
 const testdir = dirname(@__FILE__)
